--[[

RandomCompanion
Picks a random vanity pet or mount and automatically chooses the best type of mount for the job.

Usage:
Bind a key from the keybinding menu or...
"/rmount" or "/rmt" to choose a random mount
"/rmount ground" to choose a random ground mount in a flyable zone
"/rpet" or "/rpt" to choose a random vanity pet

TODO:
Config UI to disable mounts that you don't like or choose your favorite mounts more often...Coming very soon.
Add druid forms (aquatic form when swimming, travel form in combat, etc)
Check for a snowball before summoning the winter pets
Re-summon your vanity pet after zoning or landing from a flight path
Cancel druid forms before mounting
Button that shows your current pet/mount or a ? if you don't have one out
Verify that the mount/pet that is about to be summoned is indeed the one that you are about to summon, otherwise, do a reload
Add /rcreload

Changes:
1.3
Fixed bug mount prioritization in AQ40 - May not work properly in non-English clients yet.
Changed the German localization to support an oddity with the Turbogetriebene Flugmaschine.
Added /rcstatus to get RandomCompanion status and /rcdebug to toggle debug mode.
Fixed a bug with scaling flying mounts not getting added to the slow flyers list.
Uncached mounts and pets should be handled a lot better.
Fixed a bug with flyable detection inside Dalaran.

]]--

local VERSION = "1.3";

RandomCompanion = {
	mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			bugs = {};
		};
	};
	critters = {
		weighted = {};
		unweighted = {};
	};
};

local function debugmsg(msg)
	if RandomCompanion_settings.DEBUG then
		DEFAULT_CHAT_FRAME:AddMessage('RandomCompanion Debug: ' .. msg);
	end
end

function RandomCompanion.Initialize(self, event, ...)
	RC_REQUIRE_RELOAD = false;
	
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings.DEBUG then
		RandomCompanion_settings.DEBUG = false;
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	local scalingmounts = {
		ground = {};
		flying = {};
	};
	
	RandomCompanion.mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swiftground = {};
			swiftflying = {};
			extremeflying = {};
			bugs = {};
		};
	};
	RandomCompanion.critters = {
		weighted = {};
		unweighted = {};
	};
	
	CreateFrame("GameTooltip", "RandomCompanionScanningTooltip", UIParent, "GameTooltipTemplate");
	RandomCompanionScanningTooltip:AddFontStrings(
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" )
	);
	
	for index = 1, GetNumCompanions("MOUNT") do
		local aq = false;
		local outland = false;
		local northrend = false;
		local very = false;
		local extremely = false;
		local fast = false;
		local creatureid, name, spellid, icon, active = GetCompanionInfo("MOUNT", index);
		
		if name then
			GameTooltip_SetDefaultAnchor(RandomCompanionScanningTooltip, UIParent) 
			RandomCompanionScanningTooltip:SetHyperlink("spell:"..spellid)
			
			for i = 1, RandomCompanionScanningTooltip:NumLines() do
				local text = _G["RandomCompanionScanningTooltipTextLeft"..i]:GetText();
				
				if string.find(text, RC_STRING_AHN_QIRAJ) then
					aq = true;
				end
				if string.find(text, RC_STRING_OUTLAND) then
					outland = true;
				end
				if string.find(text, RC_STRING_NORTHREND) then
					northrend = true;
				end
				if string.find(text, RC_STRING_VERY) then
					very = true;
				end
				if string.find(text, RC_STRING_EXTREMELY) then
					extremely = true;
				end
				if string.find(text, RC_STRING_FAST) then
					fast = true;
				end
			end
			
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			
			local weight = RandomCompanion.GetWeight(name);
			if name == RC_STRING_BLIZZARD_BEAR then
				--This mount has to be handled separately because it scales based on skill
				--Thanks to Huntrah-Mal'ganis for helping to test this
				debugmsg("OMG! You have the Blizzcon Bear! AWESOME!");
				local nextindex = table.getn(scalingmounts.ground) + 1;
				scalingmounts.ground[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif name == RC_STRING_HORSEMAN_MOUNT then
				--This mount has to be handled separately because it scales based on location and skill
				--Thanks to Huntrah-Mal'ganis for helping to test this
				debugmsg("OMG! You have the Horseman's Mount! AWESOME!");
				local nextindex = table.getn(scalingmounts.flying) + 1;
				scalingmounts.flying[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
				local nextindex = table.getn(scalingmounts.ground) + 1;
				scalingmounts.ground[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif name == RC_STRING_DK_FLYER then
				--This mount has to be handled separately because it scales based on skill
				debugmsg("OMG! You have the Deathknight Flying Mount! AWESOME!");
				local nextindex = table.getn(scalingmounts.flying) + 1;
				scalingmounts.flying[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif aq then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.bugs) + 1;
				RandomCompanion.mounts.unweighted.bugs[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			--310% speed mounts have to be specified because most of them do not have special text to indicate they are faster than normal
			elseif (extremely and fast and (outland or northrend)) or name == RC_STRING_ASHES_OF_ALAR or name == RC_STRING_BLACK_PROTO_DRAKE or name == RC_STRING_PLAGUED_PROTO_DRAKE or name == RC_STRING_VIOLET_PROTO_DRAKE then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.extremeflying) + 1;
				RandomCompanion.mounts.unweighted.extremeflying[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif very and fast and (outland or northrend) then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftflying) + 1;
				RandomCompanion.mounts.unweighted.swiftflying[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif very and fast and not outland and not northrend then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftground) + 1;
				RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif not very and not fast and (outland or northrend) then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.flying) + 1;
				RandomCompanion.mounts.unweighted.flying[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			elseif not very and not fast and not outland and not northrend then
				local nextindex = table.getn(RandomCompanion.mounts.unweighted.ground) + 1;
				RandomCompanion.mounts.unweighted.ground[nextindex] = {
					creatureid = creatureid;
					name = name;
					index = index;
					spellid = spellid;
					icon = icon;
					active = active;
				}
			else
				-- This should never happen, but just in case...
				local options = "";
				if very then
					options = options .. " very";
				end
				if extremely then
					options = options .. " extremely";
				end
				if fast then
					options = options .. " fast";
				end
				if outland then
					options = options .. " outland";
				end
				if northrend then
					options = options .. " northrend";
				end
				if options == "" then
					options = " none";
				end
				DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion " .. VERSION .. ":" .. name .. " mount type could not be detected. (spellid:" .. spellid .. " options:" .. options .. ")");
				DEFAULT_CHAT_FRAME:AddMessage("Please leave a comment about this at http://wow.curse.com/downloads/wow-addons/details/randomcompanion.aspx");
			end
			
			if weight ~= 1 then
				debugmsg("Mount " .. name .. " has non-default weight: " .. weight);
			end
			
			if weight > 0 then
				for counter = 1, weight do
					if name == RC_STRING_HORSEMAN_MOUNT or name == RC_STRING_BLIZZARD_BEAR or name == RC_STRING_DK_FLYER then
						--do nothing here because these get added in later
					elseif aq then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.bugs) + 1;
						RandomCompanion.mounts.weighted.bugs[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					--310% speed mounts have to be specified because most of them do not have special text to indicate they are faster than normal
					elseif (extremely and fast and (outland or northrend)) or name == RC_STRING_ASHES_OF_ALAR or name == RC_STRING_BLACK_PROTO_DRAKE or name == RC_STRING_PLAGUED_PROTO_DRAKE or name == RC_STRING_VIOLET_PROTO_DRAKE then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.extremeflying) + 1;
						RandomCompanion.mounts.weighted.extremeflying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif very and fast and (outland or northrend) then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftflying) + 1;
						RandomCompanion.mounts.weighted.swiftflying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif very and fast and not outland and not northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftground) + 1;
						RandomCompanion.mounts.weighted.swiftground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif not very and not fast and (outland or northrend) then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.flying) + 1;
						RandomCompanion.mounts.weighted.flying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif not very and not fast and not outland and not northrend then
						local nextindex = table.getn(RandomCompanion.mounts.weighted.ground) + 1;
						RandomCompanion.mounts.weighted.ground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					end
				end
			end
		else
			--This seems to happen when the companion is not in the cache
			RC_REQUIRE_RELOAD = true;
			debugmsg(index .. " index mount has no name?");
		end
	end
	
	RandomCompanionScanningTooltip:Hide()
	
	--Add in the scaling flying mounts
	debugmsg("Processing scaling flying mounts");
	for key, mount in pairs(scalingmounts.flying) do
		debugmsg(mount.name .. " is a scaling flyer");
		if table.getn(RandomCompanion.mounts.unweighted.swiftflying) > 0 then
			debugmsg("Adding " .. mount.name .. " to unweighted swiftflying.");
			local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftflying) + 1;
			RandomCompanion.mounts.unweighted.swiftflying[nextindex] = {
				creatureid = mount.creatureid;
				name = mount.name;
				index = mount.index;
				spellid = mount.spellid;
				icon = mount.icon;
				active = mount.active;
			};
		end
		if table.getn(RandomCompanion.mounts.unweighted.flying) > 0 or mount.name == RC_STRING_DK_FLYER then
			debugmsg("Adding " .. mount.name .. " to unweighted flying.");
			local nextindex = table.getn(RandomCompanion.mounts.unweighted.flying) + 1;
			RandomCompanion.mounts.unweighted.flying[nextindex] = {
				creatureid = mount.creatureid;
				name = mount.name;
				index = mount.index;
				spellid = mount.spellid;
				icon = mount.icon;
				active = mount.active;
			};
		end
		
		local weight = RandomCompanion.GetWeight(mount.name);
		
		if weight > 0 then
			for counter = 1, weight do
				if table.getn(RandomCompanion.mounts.weighted.swiftflying) > 0 then
					debugmsg("Adding " .. mount.name .. " to weighted swiftflying.");
					local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftflying) + 1;
					RandomCompanion.mounts.weighted.swiftflying[nextindex] = {
						creatureid = mount.creatureid;
						name = mount.name;
						index = mount.index;
						spellid = mount.spellid;
						icon = mount.icon;
						active = mount.active;
					}
				end
				if table.getn(RandomCompanion.mounts.weighted.flying) > 0 or mount.name == RC_STRING_DK_FLYER then
					debugmsg("Adding " .. mount.name .. " to weighted flying.");
					local nextindex = table.getn(RandomCompanion.mounts.weighted.flying) + 1;
					RandomCompanion.mounts.weighted.flying[nextindex] = {
						creatureid = mount.creatureid;
						name = mount.name;
						index = mount.index;
						spellid = mount.spellid;
						icon = mount.icon;
						active = mount.active;
					}
				end
			end
		end
	end
	
	--Add in the scaling ground mounts
	for key, mount in pairs(scalingmounts.ground) do
		if table.getn(RandomCompanion.mounts.unweighted.swiftground) > 0 then
			local nextindex = table.getn(RandomCompanion.mounts.unweighted.swiftground) + 1;
			RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
				creatureid = mount.creatureid;
				name = mount.name;
				index = mount.index;
				spellid = mount.spellid;
				icon = mount.icon;
				active = mount.active;
			};
		end
		local nextindex = table.getn(RandomCompanion.mounts.unweighted.ground) + 1;
		RandomCompanion.mounts.unweighted.swiftground[nextindex] = {
			creatureid = mount.creatureid;
			name = mount.name;
			index = mount.index;
			spellid = mount.spellid;
			icon = mount.icon;
			active = mount.active;
		};
		
		local weight = RandomCompanion.GetWeight(mount.name);
		
		if weight > 0 then
			for counter = 1, weight do
				if table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
					local nextindex = table.getn(RandomCompanion.mounts.weighted.swiftground) + 1;
					RandomCompanion.mounts.weighted.swiftground[nextindex] = {
						creatureid = mount.creatureid;
						name = mount.name;
						index = mount.index;
						spellid = mount.spellid;
						icon = mount.icon;
						active = mount.active;
					}
				end
				local nextindex = table.getn(RandomCompanion.mounts.weighted.ground) + 1;
				RandomCompanion.mounts.weighted.ground[nextindex] = {
					creatureid = mount.creatureid;
					name = mount.name;
					index = mount.index;
					spellid = mount.spellid;
					icon = mount.icon;
					active = mount.active;
				}
			end
		end
	end
	
	for index = 1, GetNumCompanions("CRITTER") do
		local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", index);
		if name then
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			local weight = RandomCompanion.GetWeight(name);
			
			local nextindex = table.getn(RandomCompanion.critters.unweighted) + 1;
			RandomCompanion.critters.unweighted[nextindex] = {
				creatureid = creatureid;
				name = name;
				index = index;
				spellid = spellid;
				icon = icon;
				active = active;
			}
			
			if weight ~= 1 then
				debugmsg("Critter " .. name .. " has non-default weight: " .. weight);
			end
			
			if weight > 0 then
				for counter = 1, weight do
					local nextindex = table.getn(RandomCompanion.critters.weighted) + 1;
					RandomCompanion.critters.weighted[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				end
			end
		else
			--This seems to happen when the companion is not in the cache
			RC_REQUIRE_RELOAD = true;
			debugmsg(index .. " index critter has no name?");
		end
	end
	
	if event == "PLAYER_LOGIN" then
		RandomCompanion.Status();
	end
end

function RandomCompanion.Debug()
	if RandomCompanion_settings.DEBUG then
		RandomCompanion_settings.DEBUG = false;
		DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion debug disabled");
	else
		RandomCompanion_settings.DEBUG = true;
		DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion debug enabled");
	end
end

function RandomCompanion.Status()
	local loadmessage = "";
	
	if table.getn(RandomCompanion.mounts.unweighted.extremeflying) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.extremeflying) .. " extreme flyers";
	end
	if table.getn(RandomCompanion.mounts.unweighted.swiftflying) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.swiftflying) .. " swift flyers";
	end
	if table.getn(RandomCompanion.mounts.unweighted.flying) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.flying) .. " slow flyers";
	end
	if table.getn(RandomCompanion.mounts.unweighted.swiftground) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.swiftground) .. " swift grounders";
	end
	if table.getn(RandomCompanion.mounts.unweighted.ground) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.ground) .. " slow grounders";
	end
	if table.getn(RandomCompanion.mounts.unweighted.bugs) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.mounts.unweighted.bugs) .. " bugs";
	end
	if table.getn(RandomCompanion.critters.unweighted) > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. table.getn(RandomCompanion.critters.unweighted) .. " critters";
	end
	
	if loadmessage ~= "" then
		loadmessage = " with " .. loadmessage;
	end
	
	if RC_REQUIRE_RELOAD then
		loadmessage = loadmessage .. ". RandomCompanion will reload due to uncached companion names.";
	end
	
	DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion " .. VERSION .. " loaded" .. loadmessage);
end

function RandomCompanion.Mount(cmd)
	local flyable;
	
	if RC_REQUIRE_RELOAD then
		--Re-initialize if one of the pet or mount names was not available in the initial load
		debugmsg("Re-initializing RandomCompanion");
		RandomCompanion.Initialize();
	end
	
	if IsMounted() then
		Dismount();
	else
		if GetRealZoneText() == RC_STRING_TEMPLE_OF_AHN_QIRAJ then
			if table.getn(RandomCompanion.mounts.weighted.bugs) > 0 then
				mountlist = "bugs";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		elseif RandomCompanion.IsFlyableArea() and cmd ~= "ground" then
			if table.getn(RandomCompanion.mounts.weighted.extremeflying) > 0 then
				mountlist = "extremeflying";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftflying) > 0 then
				mountlist = "swiftflying";
			elseif table.getn(RandomCompanion.mounts.weighted.flying) > 0 then
				mountlist = "flying";
			elseif table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		else
			if table.getn(RandomCompanion.mounts.weighted.swiftground) > 0 then
				mountlist = "swiftground";
			elseif table.getn(RandomCompanion.mounts.weighted.ground) > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		end
		
		if mountlist then
			local mountindex = random(table.getn(RandomCompanion.mounts.weighted[mountlist]));
			debugmsg("Mounting " .. RandomCompanion.mounts.weighted[mountlist][mountindex].name .. " from " .. mountlist .." list. Index is " .. RandomCompanion.mounts.weighted[mountlist][mountindex].index);
			CallCompanion("MOUNT", RandomCompanion.mounts.weighted[mountlist][mountindex].index);
		else
			if GetNumCompanions("MOUNT") > 0 then
				--We have mounts, but they aren't in the mount list. The most likely issue is that no mounts were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
				if not RC_REINITIALIZED then
					debugmsg("Re-initializing because mount names weren't cached yet.");
					RC_REINITIALIZED = true; --We don't want an infinite loop, do we?
					RandomCompanion.Initialize();
					RandomCompanion.Mount(cmd);
				else
					RC_REINITIALIZED = nil;
				end
			else
				UIErrorsFrame:AddMessage("You have no mounts", 1.0, 0.0, 0.0, 53, 5);
			end
		end
	end
end

function RandomCompanion.IsFlyableArea()
	if GetRealZoneText() == RC_STRING_BOREAN_TUNDRA or GetRealZoneText() == RC_STRING_CRYSTALSONG_FOREST or GetRealZoneText() == RC_STRING_DALARAN or GetRealZoneText() == RC_STRING_DRAGONBLIGHT or GetRealZoneText() == RC_STRING_GRIZZLY_HILLS or GetRealZoneText() == RC_STRING_HOWLING_FJORD or GetRealZoneText() == RC_STRING_ICECROWN or GetRealZoneText() == RC_STRING_SHOLAZAR_BASIN or GetRealZoneText() == RC_STRING_STORM_PEAKS or GetRealZoneText() == RC_STRING_WINTERGRASP or GetRealZoneText() == RC_STRING_ZUL_DRAK then
		coldflying, outOfMana = IsUsableSpell(RC_STRING_COLD_WEATHER_FLYING);
		
		if coldflying then
			if GetRealZoneText() == RC_STRING_DALARAN then
				if GetMinimapZoneText() == RC_STRING_KRASUS_LANDING then
					return true;
				else 
					if GetMinimapZoneText() == RC_STRING_UNDERBELLY then 
						--Check coords to see if we're in the Sewer Pipe...Shamelessly stolen from Livestock
						local x, y = GetPlayerMapPosition("player")
						if x >= 0.092141166 and x <= 0.297127217055 and y >= 0.540120124816 and y <= 0.885304749012 then
							return true;
						else
							return false;
						end
					else
						return false;
					end
				end
			elseif GetRealZoneText() == RC_STRING_WINTERGRASP then
				return false;
			elseif IsFlyableArea() and coldflying then
				return true;
			else
				return false;
			end
		else
			return false;
		end
	else
		return IsFlyableArea();
	end
end

function RandomCompanion.Pet()
	if RC_REQUIRE_RELOAD then
		--Re-initialize if one of the pet or mount names was not available in the initial load
		debugmsg("Re-initializing RandomCompanion");
		RandomCompanion.Initialize();
	end
	
	if table.getn(RandomCompanion.critters.weighted) > 0 then
		local petindex = random(table.getn(RandomCompanion.critters.weighted));
		RandomCompanion.PetDismiss();
		debugmsg("Summoning " .. RandomCompanion.critters.weighted[petindex].name .. ". Index is " .. RandomCompanion.critters.weighted[petindex].index);
		CallCompanion("CRITTER", RandomCompanion.critters.weighted[petindex].index);
	else
		if GetNumCompanions("CRITTER") > 0 then
			--We have pets, but they aren't in the pet list. The most likely issue is that no mounts were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
			if not RC_REINITIALIZED then
				debugmsg("Re-initializing because pet names weren't cached yet.");
				RC_REINITIALIZED = true; --We don't want an infinite loop, do we?
				RandomCompanion.Initialize();
				RandomCompanion.Pet();
			else
				RC_REINITIALIZED = nil;
			end
		else
			UIErrorsFrame:AddMessage("You have no vanity pets", 1.0, 0.0, 0.0, 53, 5);
		end
	end
end

function RandomCompanion.PetDismiss()
	DismissCompanion("CRITTER");
end

function RandomCompanion.SetWeight(companion, value)
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] = value;
	debugmsg("RandomCompanion_settings["..GetRealmName().."]["..UnitName("player").."].weights["..companion.."] = "..RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion]);
end

function RandomCompanion.GetWeight(companion)
	if RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] then
		return RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion];
	else
		return 1;
	end
end

function RandomCompanion.Config()
	RandomCompanionConfig:Show();
end

SlashCmdList["RANDOMMOUNT"] = RandomCompanion.Mount;
SLASH_RANDOMMOUNT1 = "/randommount";
SLASH_RANDOMMOUNT2 = "/rmount";
SLASH_RANDOMMOUNT3 = "/rmt";

SlashCmdList["RANDOMPET"] = RandomCompanion.Pet;
SLASH_RANDOMPET1 = "/randompet";
SLASH_RANDOMPET2 = "/rpet";
SLASH_RANDOMPET3 = "/rpt";

SlashCmdList["PETDISMISS"] = RandomCompanion.PetDismiss;
SLASH_PETDISMISS1 = "/petdismiss";
SLASH_PETDISMISS2 = "/pdismiss";

SlashCmdList["RCSTATUS"] = RandomCompanion.Status;
SLASH_RCSTATUS1 = "/rcstatus";

SlashCmdList["RCDEBUG"] = RandomCompanion.Debug;
SLASH_RCDEBUG1 = "/rcdebug";

local frame = CreateFrame("Frame", "RandomCompanionFrame");
frame:SetScript("OnEvent", RandomCompanion.Initialize);
frame:RegisterEvent("COMPANION_LEARNED");
frame:RegisterEvent("PLAYER_LOGIN");
